#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

#  wnf_tools.py
#  
#  Copyright 2018 Uwe Wilske <wnf@wlsoft.de>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  

import sys
import os
import datetime
import humanfriendly
import configparser

def getIni(aIniDateiname):
    aIniDateiname = getHomeDirConfigWlsoft(aIniDateiname)
    print(aIniDateiname)
    ini = configparser.ConfigParser()
    ini.read(aIniDateiname)
    return ini,aIniDateiname

def saveIni(ini,aIniDateiname):
    fd = open(aIniDateiname, 'w')
    ini.write(fd)
    fd.close()  

def wnfHomeDir():
    return os.environ["HOME"]

def getHomeDirConfigWlsoft(aDateiname):
    return os.path.join(wnfHomeDir(), '.config', 'wlsoft', aDateiname)


def iniGetString(aIni, aSection, aKey):
    try:
        config = aIni[aSection]
        if config:
            s = config.get(aKey)
            if s:
                return s
            else:
                return ''
        else:
            return ''
    except Exception as E:
        print(E.args)
        return ''

def iniSetString(ini, aSection, aName, aWert):
    if (not ini.has_section(aSection)):
        ini.add_section(aSection)
    ini.set(aSection, aName, aWert)

def wnfDirExists(aPfad):
    return os.path.exists(aPfad)


def wnfFileExists(aDateiname):
    b=os.path.exists(aDateiname)
    if (b):
       b=os.path.isfile(aDateiname)
    return b

def wnfFileSizeToStr(dn):
    num_bytes = os.path.getsize(dn)
    return humanfriendly.format_size(num_bytes)

def wnfFileTimeToStr(dn):
    t = os.path.getmtime(dn)
    d = datetime.datetime.fromtimestamp(t)
    return d.strftime('%Y-%m-%d %H:%M:%S')
        
def printFNF(dn):
    print('Die Datei %s existiert nicht.' % dn)

def printDateigroesseZeit(dn):
    if os.path.isfile(dn):
        # print('%60s %s %s' % (dn,wnfFileSizeToStr(dn),wnfFileTimeToStr(dn)))
        print("{:<60s}{:>10s}{:>20s}".format(dn,wnfFileSizeToStr(dn),wnfFileTimeToStr(dn)))
    else:
        printFNF(dn)

def main():
    print(sys.version)
    printDateigroesseZeit('/wnfdaten/datensicherungen/wnf_KUB.gbk')
    return 0


if __name__ == '__main__':
    main()

