import paramiko
import os
import configparser
from stat import S_ISDIR, S_ISREG

cIni = 'wnfFTP.ini'
cScholli = 'scholli'
cNiederfohna = 'niederfrohna'
cWlsoft = 'wlsoft'
cWlsoftDownloads = 'WlsoftDownloads'

cWWW_wlsoft_de_downloads = '/WWWROOT/201710/htdocs'

def iniGetString(aIni, aSection, aKey):
    try:
        config = aIni[aSection]
        if config:
            s = config.get(aKey)
            if s:
                return s
            else:
                return ''
        else:
            return ''
    except Exception as E:
        print(E.args)
        return ''


def leseIni(aSection):
    aIniDateiname = os.path.join(os.environ["HOME"], '.config', 'wlsoft', cIni)
    print(aIniDateiname)
    aIni = configparser.ConfigParser()
    aIni.read(aIniDateiname)
    aHost = iniGetString(aIni, aSection, 'FTPHost')
    aUser = iniGetString(aIni, aSection, 'FTPUser')
    aPass = iniGetString(aIni, aSection, 'FTPPass')
    return aHost, aUser, aPass


def forceDirX(sftp, remote_directory):
    """
    https://stackoverflow.com/questions/14819681/upload-files-using-sftp-in-python-but-create-directories-if-path-doesnt-exist
    """
    """Change to this directory, recursively making new folders if needed.
    Returns True if any folders were created."""
    if remote_directory == '/':
        # absolute path so change directory to root
        sftp.chdir('/')
        return
    if remote_directory == '':
        # top-level relative directory must exist
        return
    try:
        sftp.chdir(remote_directory)  # sub-directory exists
    except IOError:
        dirname, basename = os.path.split(remote_directory.rstrip('/'))
        forceDirX(sftp, dirname)  # make parent directories
        sftp.mkdir(basename)  # sub-directory missing, so created it
        sftp.chdir(basename)
        return True


def isDir(sftp, aDir):
    for entry in sftp.listdir_attr('.'):
        if entry.filename == aDir:
            mode = entry.st_mode
            if S_ISDIR(mode):
                # print(entry.filename + " is folder")
                return True
            else:
                # print(entry.filename + " is file")
                return False
    return False


def changeDir(sftp, remote_path):
    s = remote_path.split('/')
    # print(s)
    for t in s:
        if t != '':
            # print('changeDir ' + t)
            if not isDir(sftp, t):
                return False
            sftp.chdir(t)
    return True


def forceDir(sftp, remote_path):
    s = remote_path.split('/')
    # print(s)
    for t in s:
        if t != '':
            # print('changeDir ' + t)
            if not isDir(sftp, t):
                print('makeDir ' + t)
                sftp.mkdir(t)
            sftp.chdir(t)
    return True


def verbinden(ssh, aSection):
    aHost, aUser, aPass = leseIni(aSection)
    ssh.load_system_host_keys()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(aHost, username=aUser, password=aPass)
    sftp = ssh.open_sftp()
    return sftp


def uploadDatei(aSection, aQuelldatei, aZielVerzeichnis):
    with paramiko.SSHClient() as ssh:
        sftp = verbinden(ssh, aSection)
        forceDir(sftp, aZielVerzeichnis)
        print("FTP-Upload %s" % aQuelldatei)
        sftp.put(aQuelldatei, os.path.basename(aQuelldatei))


def uploadVerzeichnis(aSection, aQuellVerzeichnis, aZielVerzeichnis):
    with paramiko.SSHClient() as ssh:
        sftp = verbinden(ssh, aSection)

        forceDir(sftp, aZielVerzeichnis)

        y = os.listdir(aQuellVerzeichnis)
        y.sort()
        for aDateiname in y:
            aQuelldatei = os.path.join(aQuellVerzeichnis, aDateiname)
            # print(aLocalDatei)
            print("FTP-Upload %s" % aDateiname)
            sftp.put(aQuelldatei, os.path.basename(aDateiname))


def listVerzeichnis(aSection, aZielverzeichnis):
    print(aSection)
    print(aZielverzeichnis)
    with paramiko.SSHClient() as ssh:
        sftp = verbinden(ssh, aSection)

        if changeDir(sftp, aZielverzeichnis):
            for entry in sftp.listdir_attr('.'):
                mode = entry.st_mode
                if S_ISDIR(mode):
                    print(entry.filename + " is folder")
                elif S_ISREG(mode):
                    print(entry.filename + " is file")
        else:
            print('Das Verzeichnis existiert nicht: %s' % aZielverzeichnis)


def listVerzeichnisR(sftp, aKennung):
    for entry in sftp.listdir_attr('.'):
        mode = entry.st_mode
        if S_ISDIR(mode):
            print(aKennung + '/' + entry.filename + " is folder")
            sftp.chdir(entry.filename)
            aKennung = aKennung + '/' + entry.filename
            listVerzeichnisR(sftp, aKennung)
            sftp.chdir('..')
        elif S_ISREG(mode):
            print(aKennung + '/' + entry.filename)


def listDogadoSubDomains(aSection, aZielverzeichnis):
    print(aSection)
    print(aZielverzeichnis)
    with paramiko.SSHClient() as ssh:
        sftp = verbinden(ssh, aSection)
        if changeDir(sftp, aZielverzeichnis):
            listVerzeichnisR(sftp, aZielverzeichnis)
        else:
            print('Das Verzeichnis existiert nicht: %s' % aZielverzeichnis)


def uploadToDownloads(aQuellDatei):
    aZielVerzeichnis = cWWW_wlsoft_de_downloads
    uploadDatei(cWlsoft, aQuellDatei, aZielVerzeichnis)
    # listVerzeichnis(cWlsoft, aZielVerzeichnis)


def testScholli():
    aQuellVerzeichnis = '/var/www/html/niederfrohna_de/sites/default/files/bilder/2023/baumpflanzen_2023/'
    aZielVerzeichnis = '/WWWROOT/263989/htdocs/sites/default/files/bilder/2024/zzz/'
    # uploadVerzeichnis(cScholli, aQuellVerzeichnis, aZielVerzeichnis)
    listVerzeichnis(cScholli, aZielVerzeichnis)


def testNiederfrohna():
    aQuellVerzeichnis = '/var/www/html/niederfrohna_de/sites/default/files/bilder/2023/baumpflanzen_2023/'
    aZielVerzeichnis = '/sites/default/files/bilder/2023'
    # uploadVerzeichnis(cNiederfohna, aQuellVerzeichnis, aZielVerzeichnis)
    listVerzeichnis(cNiederfohna, aZielVerzeichnis)


def testWlsoft():
    aModus = 2
    if aModus == 1:
        aZielVerzeichnis = cWWW_wlsoft_de_downloads
        listDogadoSubDomains(cWlsoft, aZielVerzeichnis)
    elif aModus == 2:
        aQuellVerzeichnis = '/home/wnf/Temp/demo_wnfjpeg_verzeichnis/baum_2023/'
        aZielVerzeichnis = cWWW_wlsoft_de_downloads
        uploadVerzeichnis(cWlsoft, aQuellVerzeichnis, aZielVerzeichnis)
    elif aModus == 3:
        aZielVerzeichnis = cWWW_wlsoft_de_downloads
        listVerzeichnis(cWlsoft, aZielVerzeichnis)


def main():
    testNiederfrohna()
    # testScholli()
    # testWlsoft()
    # testUploadToDownloads()
    return 0


if __name__ == "__main__":
    main()
